********************************************************************************
* FORECASTS IN TIMES OF CRISES -- MONA analysis:
********************************************************************************

cd "insert folder"

* Loading in MONA forecast errors and preparing dataset for regressions: 
set more off
use MONA_fcasts, clear
keep arrangementnumber arrangementtype countryname mneumonic description startyear ///
fcast_error_t_pct startmonth
save MONA_helper, replace

local vars="BCA BE BER BF BFD BFDI BFOL_G BFP BFPLBN BFRA BI BK BKFA BMG BMII BMS " ///
		 + "BTR BTRG BXG BXS D D_B D_G D_S D_SP D_SPG FCG FFA FFS FGS FMB FPO FPS " ///
		 + "GCEI GCRG GGB GGD GGENL GGRG NCG NCP NFI NFIG NGDP NGDP_R NGS NGSG NINV " ///
		 + "NM NMG NMG_R NX NXG NXG_R PCPI PCPIE BFO BFUND BIP BOP D_L D_O D_SI BFL_D " ///
		 + "BFOA BFOL BFOL_O BFPA BFPAG ENDA FBA FIR GCENL GGEI NFIP NI NMS NMS_R NXS " ///
		 + "NXS_R BED BFOL_B BFOL_G_L GCND D_A GGND BEA BFOL_L BFPLE GCD BFOL_S BFOAG " ///
		 + "GCNTR GCTR BSDS FDS GCCENL GCCNE GCG GCCR ENDE BERB BERP NXG_R BFING BFOTH " ///
		 + "BFPLD_G BFPLE_G BFPCB DB DG DO DS DSI DSP DSPG DA"

foreach k in `vars' {
use MONA_helper, clear
keep if mneumonic=="`k'"
keep arrangementnumber arrangementtype countryname fcast_error_t_pct startmonth startyear 
rename fcast_error_t_pct `k'
if("`k'"=="BCA") {
save MONA_regression, replace
}
else {
merge 1:1 arrangementnumber countryname using MONA_regression
drop _merge
save MONA_regression, replace
}
}

* Identifying LICs:
aorder
order arrangementnumber countryname
gen LIC=0
replace LIC=1 if arrangementtype=="PRGF"|arrangementtype=="PRGF-EFF"|arrangementtype=="PSI"| ///
arrangementtype=="SCF"|arrangementtype=="SBA-SCF"|arrangementtype=="ECF"|arrangementtype=="ECF-EFF"
save MONA_regression, replace
erase MONA_helper.dta
erase MONA_fcasts.dta


***************************************************************************************
* Table 1: Mincer-Zarnowitz Regressions for GDP growth and Its Subcomponents
***************************************************************************************
* Preparing NGDP data:
use MONA_cleaned, clear
keep if mneumonic=="NGDP"|mneumonic=="NCP"|mneumonic=="NM"|mneumonic=="NX"|mneumonic=="NCG"| ///
mneumonic=="NFIG"|mneumonic=="NFIP"|mneumonic=="NINV"|mneumonic=="NGDP_R"|mneumonic=="PCPI"| ///
|mneumonic=="PCPIE"
gen pct_final_t=(t_end-tminus1_end)/tminus1_end if months>=18
gen pct_first_t=(t-tminus1)/tminus1 if months>=18
keep arrangementnumber countryname mneumonic pct_final_t pct_first_t startyear crisis08 ///
america africa asia europe
save MONA_NGDP, replace

local vars="NGDP NCP NM NX NCG NFIG NFIP NINV NGDP_R PCPI"
foreach v in `vars' {
use MONA_NGDP, clear
if("`v'"=="NGDP") {
keep if mneumonic=="`v'"
drop mneumonic
rename pct_final_t `v'_actual
rename pct_first_t `v'_forecast
save NGDP_actual_forecast, replace
}
else {
keep if mneumonic=="`v'"
drop mneumonic
rename pct_final_t `v'_actual
rename pct_first_t `v'_forecast
merge 1:1 arrangementnumber countryname using NGDP_actual_forecast
drop _merge
save NGDP_actual_forecast, replace
}
}

* Removing 4sd outliers:
local vars="NGDP NCP NM NX NCG NFIG NFIP"
use MONA_regression, clear
foreach v in `vars' {
quietly: sum `v'
gen _mean=r(mean)
gen _sd=r(sd)
gen min_mean=_mean-4*_sd
gen max_mean=_mean+4*_sd
drop if `v'==.
drop if `v'<min_mean|`v'>max_mean
drop _mean _sd min_mean max_mean
}

* LIC/Non-LIC selection:
*keep if LIC==0

* Merge regression samples with MONA data forecasts and actual values:
keep arrangementnumber countryname startmonth
merge 1:1 arrangementnumber countryname using NGDP_actual_forecast
keep if _merge==3
drop _merge

* Run Mincer-Zarnowitz regressions for each GDP component:
local vars="NGDP_R PCPI NGDP NCP NCG NM NX NFIG NFIP"
local counter=0
foreach v in `vars' {
local counter=`counter'+1
rename `v'_forecast forecast
* F-test:
reg `v'_actual forecast crisis08 africa america asia, robust
est store NGDP`counter'
test (_b[_cons]=0) (_b[forecast]=1)
local Fval=r(F)
local pval_F=r(p)
reg `v'_actual forecast crisis08 africa america asia, robust
* Holden-Peel test:
gen ferror=`v'_actual-forecast
reg ferror crisis08 africa america asia, robust
mat cov=e(V)
mat b=e(b)
local b1=b[1,5]
local varb1=cov[5,5]
local seb1=sqrt(`varb1')
local tstatb1=`b1'/`seb1'
local pvalb1=tprob(e(df_r), abs(`tstatb1'))
drop ferror
* Results output:
outreg2 [NGDP`counter'] using MZreg_NGDP.xls, adds(F-test (a = 0 & b = 1), `Fval', ///
p-val (a = 0 & b = 1), `pval_F', Holden-Peel t-Test, `tstatb1', p-val, `pvalb1') excel bdec(3) noaster
rename forecast `v'_forecast
}

* Erasing generated files:
erase MZreg_NGDP.xls
erase MZreg_NGDP.txt
erase NGDP_actual_forecast.dta
erase MONA_NGDP.dta


***************************************************************************************
* Table 2: Correct and Incorrect Forecasts of GDP Growth and Its Subcomponents 
***************************************************************************************
* Prepare MONA data (forecasts and actual growth rates) for merge with regression samples:
use MONA_cleaned, clear
keep if mneumonic=="NGDP"|mneumonic=="NCP"|mneumonic=="NM"|mneumonic=="NX"|mneumonic=="NCG"| ///
mneumonic=="NFIG"|mneumonic=="NFIP"|mneumonic=="NINV"|mneumonic=="NGDP_R"|mneumonic=="PCPI"| ///
|mneumonic=="PCPIE"
gen pct_final_t=(t_end-tminus1_end)/tminus1_end if months>=18
gen pct_first_t=(t-tminus1)/tminus1 if months>=18
keep arrangementnumber countryname mneumonic pct_final_t pct_first_t
save MONA_NGDP, replace

local vars="NGDP NCP NM NX NCG NFIG NFIP NINV NGDP_R PCPI PCPIE"
foreach v in `vars' {
use MONA_NGDP, clear
if("`v'"=="NGDP") {
keep if mneumonic=="`v'"
drop mneumonic
rename pct_final_t `v'_actual
rename pct_first_t `v'_forecast
save NGDP_actual_forecast, replace
}
else {
keep if mneumonic=="`v'"
drop mneumonic
rename pct_final_t `v'_actual
rename pct_first_t `v'_forecast
merge 1:1 arrangementnumber countryname using NGDP_actual_forecast
drop _merge
save NGDP_actual_forecast, replace
}
}

* Removing 4sd outliers:
local vars="NGDP NCP NM NX NCG NFIG NFIP"
use MONA_regression, clear
foreach v in `vars' {
quietly: sum `v'
gen _mean=r(mean)
gen _sd=r(sd)
gen min_mean=_mean-4*_sd
gen max_mean=_mean+4*_sd
drop if `v'==.
drop if `v'<min_mean|`v'>max_mean
drop _mean _sd min_mean max_mean
}
* LIC/Non-LIC selection:
*keep if LIC==0
save NGDPreghelper, replace

* Forecast vs. actual comparisons:
local vars="NGDP_R PCPI NGDP NCP NCG NM NX NFIG NFIP"
local counter=0
foreach v in `vars' {
* Merge regression samples with MONA data forecasts and actual values:
use NGDPreghelper, clear
keep arrangementnumber countryname
merge 1:1 arrangementnumber countryname using NGDP_actual_forecast
keep if _merge==3
drop _merge

keep `v'_actual `v'_forecast
drop if `v'_actual==.
drop if `v'_forecast==.

local counter=`counter'+1 
gen counter=`counter'
gen pos_pos=0
gen pos_neg=0
gen neg_pos=0
gen neg_neg=0
replace pos_pos=1 if `v'_forecast>0 & `v'_actual>0
replace pos_neg=1 if `v'_forecast>0 & `v'_actual<=0
replace neg_pos=1 if `v'_forecast<=0 & `v'_actual>0
replace neg_neg=1 if `v'_forecast<=0 & `v'_actual<=0
egen pos_pos_sum=sum(pos_pos)
egen pos_neg_sum=sum(pos_neg)
egen neg_pos_sum=sum(neg_pos)
egen neg_neg_sum=sum(neg_neg)
gen total=pos_pos_sum+pos_neg_sum+neg_pos_sum+neg_neg_sum
gen pos_pos_perc=(pos_pos_sum/total)*100
gen pos_neg_perc=(pos_neg_sum/total)*100
gen neg_pos_perc=(neg_pos_sum/total)*100
gen neg_neg_perc=(neg_neg_sum/total)*100
keep pos_pos_sum pos_neg_sum neg_pos_sum neg_neg_sum pos_pos_perc pos_neg_perc ///
neg_pos_perc neg_neg_perc counter
gen mneumonic="`v'"
duplicates drop
order pos_pos_sum neg_neg_sum pos_neg_sum neg_pos_sum pos_pos_perc neg_neg_perc ///
pos_neg_perc neg_pos_perc counter

* Order for 2x2 contignency table Chi-Square test:
if("`v'"=="NGDP_R") {
save forecasteval, replace
}
if("`v'"!="NGDP_R") {
append using forecasteval
save forecasteval, replace
}
else {
continue
}
}
* Results output:
use forecasteval, clear
order counter mneumonic
sort counter
browse
* Erasing generated files:
erase forecasteval.dta
erase NGDPreghelper.dta
erase NGDP_actual_forecast.dta
erase MONA_NGDP.dta


***************************************************************************************
* Table 3: Contributors to GDP Growth Forecast Errors
***************************************************************************************
* Removing 4sd outliers:
local vars="NGDP NCP NM NX NCG NFIG NFIP"
use MONA_regression, clear
foreach v in `vars' {
quietly: sum `v'
gen _mean=r(mean)
gen _sd=r(sd)
gen min_mean=_mean-4*_sd
gen max_mean=_mean+4*_sd
drop if `v'==.
drop if `v'<min_mean|`v'>max_mean
drop _mean _sd min_mean max_mean
}

* LIC/Non-LIC selection:
*keep if LIC==0

* Output results:
reg NGDP NCP NCG NM NX NFIG NFIP, robust
outreg2 using MONAreg_NGDP_SD4.xls, excel bdec(3)

* Erase generated files:
erase MONAreg_NGDP_SD4.xls 
erase MONAreg_NGDP_SD4.txt 


***************************************************************************************
* Table 4a: Mincer-Zarnowitz regressions for current account balance growth and its subcomponents
***************************************************************************************
* Preparing BCA data:
use MONA_cleaned, clear
keep if mneumonic=="BCA"|mneumonic=="BMG"|mneumonic=="BXG"|mneumonic=="BMS"|mneumonic=="BXS"| ///
|mneumonic=="BTR"|mneumonic=="BI"
gen pct_final_t=(t_end-tminus1_end)/tminus1_end if months>=18
gen pct_first_t=(t-tminus1)/tminus1 if months>=18
keep arrangementnumber countryname mneumonic pct_final_t pct_first_t crisis08 america ///
asia africa europe
save MONA_BCA, replace

local vars="BCA BMG BXG BMS BXS BTR BI"
foreach v in `vars' {
use MONA_BCA, clear
if("`v'"=="BCA") {
keep if mneumonic=="`v'"
drop mneumonic
rename pct_final_t `v'_actual
rename pct_first_t `v'_forecast
save BCA_actual_forecast, replace
}
else {
keep if mneumonic=="`v'"
drop mneumonic
rename pct_final_t `v'_actual
rename pct_first_t `v'_forecast
merge 1:1 arrangementnumber countryname using BCA_actual_forecast
drop _merge
save BCA_actual_forecast, replace
}
}

* Removing 4sd outliers:
local vars="BCA BMG BXG BMS BXS BTR BI"
use MONA_regression, clear
foreach v in `vars' {
quietly: sum `v'
gen _mean=r(mean)
gen _sd=r(sd)
gen min_mean=_mean-4*_sd
gen max_mean=_mean+4*_sd
drop if `v'==.
drop if `v'<min_mean|`v'>max_mean
drop _mean _sd min_mean max_mean
}

* Remove all outliers for dependent variable after one full outlier run:
egen mean_BCA=mean(BCA)
egen sd_BCA=sd(BCA)
drop if abs(BCA)>4*sd_BCA

* LIC/Non-LIC selection:
*keep if LIC==0

* Merge regression samples with MONA data forecasts and actual values:
keep arrangementnumber countryname 
merge 1:1 arrangementnumber countryname using BCA_actual_forecast
keep if _merge==3
drop _merge

* Run Mincer-Zarnowitz regressions:
local vars="BCA BMG BXG BMS BXS BTR BI"
local counter=0
foreach v in `vars' {
local counter=`counter'+1
rename `v'_forecast forecast
* F-test:
reg `v'_actual forecast crisis08 africa america asia, robust
est store BCA`counter'
test (_b[_cons]=0) (_b[forecast]=1)
local Fval=r(F)
local pval_F=r(p)
* Holden-Peel test:
gen ferror=`v'_actual-forecast
reg ferror crisis08 africa america asia, robust
mat cov=e(V)
mat b=e(b)
local b1=b[1,5]
local varb1=cov[5,5]
local seb1=sqrt(`varb1')
local tstatb1=`b1'/`seb1'
local pvalb1=tprob(e(df_r), abs(`tstatb1'))
drop ferror
* Results output:
outreg2 [BCA`counter'] using MZreg_BCA.xls, adds(F-test (a = 0 & b = 1), `Fval', ///
p-val (a = 0 & b = 1), `pval_F', Holden-Peel t-Test, `tstatb1', p-val, `pvalb1') excel bdec(3) noaster
rename forecast `v'_forecast 
}

* Erasing generated files:
erase MZreg_BCA.xls
erase MZreg_BCA.txt
erase BCA_actual_forecast.dta
erase MONA_BCA.dta


***************************************************************************************
* Table 4b: Mincer-Zarnowitz regressions for financial account balance growth and its subcomponents
***************************************************************************************
* Preparing BF data:
use MONA_cleaned, clear
keep if mneumonic=="BF"|mneumonic=="BFD"|mneumonic=="BFRA"|mneumonic=="BFO"|mneumonic=="BFP"| ///
|mneumonic=="BFOL_G"|mneumonic=="BFOL"|mneumonic=="BFOA"
gen pct_final_t=(t_end-tminus1_end)/tminus1_end if months>=18
gen pct_first_t=(t-tminus1)/tminus1 if months>=18
keep arrangementnumber countryname mneumonic pct_final_t pct_first_t crisis08 africa ///
america asia europe
save MONA_BF, replace

local vars="BF BFD BFRA BFO BFP BFO BFOL_G BFOL BFOA"
foreach v in `vars' {
use MONA_BF, clear
if("`v'"=="BF") {
keep if mneumonic=="`v'"
drop mneumonic
rename pct_final_t `v'_actual
rename pct_first_t `v'_forecast
save BF_actual_forecast, replace
}
else {
keep if mneumonic=="`v'"
drop mneumonic
rename pct_final_t `v'_actual
rename pct_first_t `v'_forecast
merge 1:1 arrangementnumber countryname using BF_actual_forecast
drop _merge
save BF_actual_forecast, replace
}
}

* Removing 4sd outliers:
local vars="BF BFD BFRA BFO BFP"
use MONA_regression, clear
foreach v in `vars' {
quietly: sum `v'
gen _mean=r(mean)
gen _sd=r(sd)
gen min_mean=_mean-4*_sd
gen max_mean=_mean+4*_sd
drop if `v'==.
drop if `v'<min_mean|`v'>max_mean
drop _mean _sd min_mean max_mean
}

* Remove all outliers for dependent variable after one full outlier run:
egen mean_BF=mean(BF)
egen sd_BF=sd(BF)
drop if abs(BF)>4*sd_BF

* LIC/Non-LIC selection:
*keep if LIC==0

* Merge regression samples with MONA data forecasts and actual values:
keep arrangementnumber countryname
merge 1:1 arrangementnumber countryname using BF_actual_forecast
keep if _merge==3
drop _merge

* Run Mincer-Zarnowitz regressions:
local vars="BF BFD BFRA BFO BFP"
local counter=0
foreach v in `vars' {
local counter=`counter'+1
rename `v'_forecast forecast
* F-test:
reg `v'_actual forecast crisis08 africa america asia, robust
est store BF`counter'
test (_b[_cons]=0) (_b[forecast]=1)
local Fval=r(F)
local pval_F=r(p)
* Holden-Peel test:
gen ferror=`v'_actual-forecast
*reg ferror crisis08, robust
reg ferror crisis08 africa america asia, robust
mat cov=e(V)
mat b=e(b)
local b1=b[1,5]
local varb1=cov[5,5]
local seb1=sqrt(`varb1')
local tstatb1=`b1'/`seb1'
local pvalb1=tprob(e(df_r), abs(`tstatb1'))
drop ferror
outreg2 [BF`counter'] using MZreg_BF.xls, adds(F-test (a = 0 & b = 1), `Fval', p-val (a = 0 & b = 1), ///
`pval_F', Holden-Peel t-Test, `tstatb1', p-val, `pvalb1') excel bdec(3) noaster
rename forecast `v'_forecast 
}

* Erasing generated files:
erase MZreg_BF.xls
erase MZreg_BF.txt
erase BF_actual_forecast.dta
erase MONA_BF.dta


***************************************************************************************
* Table 5a: Correct and Incorrect Forecasts of Current Account Balance Growth and Its Subcomponents 
***************************************************************************************
* Prepare MONA data (forecasts and actual growth rates) for merge with regression samples:
use MONA_cleaned, clear
keep if mneumonic=="BCA"|mneumonic=="BMG"|mneumonic=="BXG"|mneumonic=="BMS"|mneumonic=="BXS"| ///
|mneumonic=="BTR"|mneumonic=="BI"
gen pct_final_t=(t_end-tminus1_end)/tminus1_end if months>=18
gen pct_first_t=(t-tminus1)/tminus1 if months>=18
keep arrangementnumber countryname mneumonic pct_final_t pct_first_t
save MONA_BCA, replace

local vars="BCA BMG BXG BMS BXS BTR BI"
foreach v in `vars' {
use MONA_BCA, clear
if("`v'"=="BCA") {
keep if mneumonic=="`v'"
drop mneumonic
rename pct_final_t `v'_actual
rename pct_first_t `v'_forecast
save BCA_actual_forecast, replace
}
else {
keep if mneumonic=="`v'"
drop mneumonic
rename pct_final_t `v'_actual
rename pct_first_t `v'_forecast
merge 1:1 arrangementnumber countryname using BCA_actual_forecast
drop _merge
save BCA_actual_forecast, replace
}
}

* Removing 4sd outliers:
local vars="BCA BMG BXG BMS BXS BTR BI"
use MONA_regression, clear
foreach v in `vars' {
sum `v'
gen _mean=r(mean)
gen _sd=r(sd)
gen min_mean=_mean-4*_sd
gen max_mean=_mean+4*_sd
drop if `v'==.
drop if `v'<min_mean|`v'>max_mean
drop _mean _sd min_mean max_mean
}

* Remove all outliers for dependent variable after one full outlier run:
egen mean_BCA=mean(BCA)
egen sd_BCA=sd(BCA)
drop if abs(BCA)>4*sd_BCA

* LIC/NOn-LIC selection:
*keep if LIC==0
save BCAreghelper, replace

* Forecast vs. actual comparisons:
local vars="BCA BMG BXG BMS BXS BTR BI"
local counter=0
foreach v in `vars' {
* Merge regression samples with MONA data forecasts and actual values:
use BCAreghelper, clear
keep arrangementnumber countryname
merge 1:1 arrangementnumber countryname using BCA_actual_forecast
keep if _merge==3
drop _merge

keep `v'_actual `v'_forecast
drop if `v'_actual==.
drop if `v'_forecast==.

local counter=`counter'+1 
gen counter=`counter'
gen pos_pos=0
gen pos_neg=0
gen neg_pos=0
gen neg_neg=0
replace pos_pos=1 if `v'_forecast>0 & `v'_actual>0
replace pos_neg=1 if `v'_forecast>0 & `v'_actual<=0
replace neg_pos=1 if `v'_forecast<=0 & `v'_actual>0
replace neg_neg=1 if `v'_forecast<=0 & `v'_actual<=0
egen pos_pos_sum=sum(pos_pos)
egen pos_neg_sum=sum(pos_neg)
egen neg_pos_sum=sum(neg_pos)
egen neg_neg_sum=sum(neg_neg)
gen total=pos_pos_sum+pos_neg_sum+neg_pos_sum+neg_neg_sum
gen pos_pos_perc=(pos_pos_sum/total)*100
gen pos_neg_perc=(pos_neg_sum/total)*100
gen neg_pos_perc=(neg_pos_sum/total)*100
gen neg_neg_perc=(neg_neg_sum/total)*100
keep pos_pos_sum pos_neg_sum neg_pos_sum neg_neg_sum pos_pos_perc pos_neg_perc ///
neg_pos_perc neg_neg_perc counter
gen mneumonic="`v'"
duplicates drop

* Order for 2x2 contignency table Chi-Square test:
order pos_pos_sum neg_neg_sum pos_neg_sum neg_pos_sum pos_pos_perc neg_neg_perc ///
pos_neg_perc neg_pos_perc counter

if("`v'"=="BCA") {
save forecasteval, replace
}
if("`v'"!="BCA") {
append using forecasteval
save forecasteval, replace
}
else {
continue
}
}
* Output results:
use forecasteval, clear
order counter mneumonic
sort counter
browse
* Erasing generated files:
erase forecasteval.dta
erase BCAreghelper.dta
erase BCA_actual_forecast.dta
erase MONA_BCA.dta


***************************************************************************************
* Table 5b: Correct and Incorrect Forecasts of Financial Account Balance Growth and Its Subcomponents 
***************************************************************************************
* Prepare MONA data (forecasts and actual growth rates) for merge with regression samples:
use MONA_cleaned, clear
keep if mneumonic=="BF"|mneumonic=="BFD"|mneumonic=="BFRA"|mneumonic=="BFO"|mneumonic=="BFP"| ///
|mneumonic=="BFOL_G"|mneumonic=="BFOL"|mneumonic=="BFOA"
gen pct_final_t=(t_end-tminus1_end)/tminus1_end if months>=18
gen pct_first_t=(t-tminus1)/tminus1 if months>=18
keep arrangementnumber countryname mneumonic pct_final_t pct_first_t
save MONA_BF, replace

local vars="BF BFD BFRA BFP BFO BFOL_G BFOL BFOA"
foreach v in `vars' {
use MONA_BF, clear
if("`v'"=="BF") {
keep if mneumonic=="`v'"
drop mneumonic
rename pct_final_t `v'_actual
rename pct_first_t `v'_forecast
save BF_actual_forecast, replace
}
else {
keep if mneumonic=="`v'"
drop mneumonic
rename pct_final_t `v'_actual
rename pct_first_t `v'_forecast
merge 1:1 arrangementnumber countryname using BF_actual_forecast
drop _merge
save BF_actual_forecast, replace
}
}

* Removing 4sd outliers:
local vars="BF BFD BFRA BFO BFP"
use MONA_regression, clear
foreach v in `vars' {
sum `v'
gen _mean=r(mean)
gen _sd=r(sd)
gen min_mean=_mean-4*_sd
gen max_mean=_mean+4*_sd
drop if `v'==.
drop if `v'<min_mean|`v'>max_mean
drop _mean _sd min_mean max_mean
}
* Remove all outliers for dependent variable after one full outlier run:
egen mean_BF=mean(BF)
egen sd_BF=sd(BF)
drop if abs(BF)>4*sd_BF

* LIC/Non-LIC selection:
*keep if LIC==0
save BFreghelper, replace

* Forecast vs. actual comparisons:
local vars="BF BFD BFRA BFO BFP"
local counter=0
foreach v in `vars' {
* Merge regression samples with MONA data forecasts and actual values:
use BFreghelper, clear
keep arrangementnumber countryname
merge 1:1 arrangementnumber countryname using BF_actual_forecast
keep if _merge==3
drop _merge

keep `v'_actual `v'_forecast
drop if `v'_actual==.
drop if `v'_forecast==.

local counter=`counter'+1 
gen counter=`counter'
gen pos_pos=0
gen pos_neg=0
gen neg_pos=0
gen neg_neg=0
replace pos_pos=1 if `v'_forecast>0 & `v'_actual>0
replace pos_neg=1 if `v'_forecast>0 & `v'_actual<=0
replace neg_pos=1 if `v'_forecast<=0 & `v'_actual>0
replace neg_neg=1 if `v'_forecast<=0 & `v'_actual<=0
egen pos_pos_sum=sum(pos_pos)
egen pos_neg_sum=sum(pos_neg)
egen neg_pos_sum=sum(neg_pos)
egen neg_neg_sum=sum(neg_neg)
gen total=pos_pos_sum+pos_neg_sum+neg_pos_sum+neg_neg_sum
gen pos_pos_perc=(pos_pos_sum/total)*100
gen pos_neg_perc=(pos_neg_sum/total)*100
gen neg_pos_perc=(neg_pos_sum/total)*100
gen neg_neg_perc=(neg_neg_sum/total)*100
keep pos_pos_sum pos_neg_sum neg_pos_sum neg_neg_sum pos_pos_perc pos_neg_perc ///
neg_pos_perc neg_neg_perc counter
gen mneumonic="`v'"
duplicates drop

* Order for 2x2 contignency table Chi-Square test:
order pos_pos_sum neg_pos_sum pos_neg_sum neg_neg_sum pos_pos_perc neg_neg_perc ///
pos_neg_perc neg_pos_perc counter

if("`v'"=="BF") {
save forecasteval, replace
}
if("`v'"!="BF") {
append using forecasteval
save forecasteval, replace
}
else {
continue
}
}
* Output results:
use forecasteval, clear
order counter mneumonic
sort counter
browse
* Erasing generated files:
erase forecasteval.dta
erase BFreghelper.dta
erase BF_actual_forecast.dta
erase MONA_BF.dta


***************************************************************************************
* Table 6: Contributors to Balance of Payments Forecast Errors
***************************************************************************************
* Panel A: Current Account Balance
**********************************
* Removing 4sd outliers:
local vars="BCA BMG BXG BMS BXS BTR BI"
use MONA_regression, clear
foreach v in `vars' {
quietly: sum `v'
gen _mean=r(mean)
gen _sd=r(sd)
gen min_mean=_mean-4*_sd
gen max_mean=_mean+4*_sd
drop if `v'==.
drop if `v'<min_mean|`v'>max_mean
drop _mean _sd min_mean max_mean
}

* Remove all outliers for dependent variable after one full outlier run:
egen mean_BCA=mean(BCA)
egen sd_BCA=sd(BCA)
drop if abs(BCA)>4*sd_BCA

* LIC/Non-LIC selection:
*keep if LIC==0

* Output results:
reg BCA BMG BXG BMS BXS BTR BI, robust
outreg2 using MONAreg_BCA_SD4.xls, excel bdec(3)

* Erase generated files:
erase MONAreg_BCA_SD4.xls 
erase MONAreg_BCA_SD4.txt 

* Panel B: Financial Account Balance
************************************
* Removing 4sd outliers:
local vars="BF BFD BFRA BFO BFP"
use MONA_regression, clear
foreach v in `vars' {
quietly: sum `v'
gen _mean=r(mean)
gen _sd=r(sd)
gen min_mean=_mean-4*_sd
gen max_mean=_mean+4*_sd
drop if `v'==.
drop if `v'<min_mean|`v'>max_mean
drop _mean _sd min_mean max_mean
}

* Remove all outliers for dependent variable after one full outlier run:
egen mean_BF=mean(BF)
egen sd_BF=sd(BF)
drop if abs(BF)>4*sd_BF

* LIC/Non-LIC selection:
*keep if LIC==0

* Output results:
reg BF BFD BFRA BFO BFP, robust
outreg2 using MONAreg_BF_SD4.xls, excel bdec(3)

* Erase generated files:
erase MONAreg_BF_SD4.xls 
erase MONAreg_BF_SD4.txt 


***************************************************************************************
* Table 7a: Mincer-Zarnowitz regressions for government expenditure growth and its subcomponents
***************************************************************************************
* Preparing GCENL data:
use MONA_cleaned, clear
keep if mneumonic=="GCENL"|mneumonic=="GCEI"|mneumonic=="GCCNE"|mneumonic=="GCCENL"
gen pct_final_t=(t_end-tminus1_end)/tminus1_end if months>=18
gen pct_first_t=(t-tminus1)/tminus1 if months>=18
keep arrangementnumber countryname mneumonic pct_final_t pct_first_t crisis08 america ///
asia africa europe
save MONA_GCENL, replace

local vars="GCENL GCEI GCCNE GCCENL"
foreach v in `vars' {
use MONA_GCENL, clear
if("`v'"=="GCENL") {
keep if mneumonic=="`v'"
drop mneumonic
rename pct_final_t `v'_actual
rename pct_first_t `v'_forecast
save GCENL_actual_forecast, replace
}
else {
keep if mneumonic=="`v'"
drop mneumonic
rename pct_final_t `v'_actual
rename pct_first_t `v'_forecast
merge 1:1 arrangementnumber countryname using GCENL_actual_forecast
drop _merge
save GCENL_actual_forecast, replace
}
}

* Removing 4sd outliers:
local vars="GCENL GCEI GCCNE GCCENL"
use MONA_regression, clear
foreach v in `vars' {
quietly: sum `v'
gen _mean=r(mean)
gen _sd=r(sd)
gen min_mean=_mean-4*_sd
gen max_mean=_mean+4*_sd
drop if `v'==.
drop if `v'<min_mean|`v'>max_mean
drop _mean _sd min_mean max_mean
}

* Remove all outliers for dependent variable after one full outlier run:
egen mean_GCENL=mean(GCENL)
egen sd_GCENL=sd(GCENL)
drop if abs(GCENL)>4*sd_GCENL

* LIC/Non-LIC selection:
*keep if LIC==1
gen obs=_N
egen LICcount=sum(LIC)

* Merge regression samples with MONA data forecasts and actual values:
keep arrangementnumber countryname obs LICcount
merge 1:1 arrangementnumber countryname using GCENL_actual_forecast
keep if _merge==3
drop _merge

* Run Mincer-Zarnowitz regressions:
local vars="GCENL GCEI GCCNE GCCENL"
local counter=0
foreach v in `vars' {
local counter=`counter'+1
rename `v'_forecast forecast 
* F-test:
if(LICcount[1]==obs[1]) {
reg `v'_actual forecast crisis08 africa america, robust 
}
else {
reg `v'_actual forecast crisis08 africa america asia, robust 
}
est store GCENL`counter'
test (_b[_cons]=0) (_b[forecast]=1)
local Fval=r(F)
local pval_F=r(p)
* Holden-Peel test:
gen ferror=`v'_actual-forecast
if(LICcount[1]==obs[1]) {
reg ferror crisis08 africa america, robust
mat cov=e(V)
mat b=e(b)
local b1=b[1,4]
local varb1=cov[4,4]
}
else {
reg ferror crisis08 africa america asia, robust
mat cov=e(V)
mat b=e(b)
local b1=b[1,5]
local varb1=cov[5,5]
}
local seb1=sqrt(`varb1')
local tstatb1=`b1'/`seb1'
local pvalb1=tprob(e(df_r), abs(`tstatb1'))
drop ferror
* Results output:
outreg2 [GCENL`counter'] using MZreg_GCENL.xls, adds(F-test (a = 0 & b = 1), `Fval', ///
p-val (a = 0 & b = 1), `pval_F', Holden-Peel t-Test, `tstatb1', p-val, `pvalb1') excel bdec(3) noaster
rename forecast `v'_forecast 
}

* Erasing generated files:
erase MZreg_GCENL.xls
erase MZreg_GCENL.txt
erase GCENL_actual_forecast.dta
erase MONA_GCENL.dta


***************************************************************************************
* Table 7b: Mincer-Zarnowitz regressions for government revenue growth and its subcomponents
***************************************************************************************
* Preparing GCRG data:
use MONA_cleaned, clear
keep if mneumonic=="GCRG"|mneumonic=="GCG"|mneumonic=="GCTR"|mneumonic=="GCNTR"
gen pct_final_t=(t_end-tminus1_end)/tminus1_end if months>=18
gen pct_first_t=(t-tminus1)/tminus1 if months>=18
keep arrangementnumber countryname mneumonic pct_final_t pct_first_t crisis08 america ///
africa asia europe
save MONA_GCRG, replace

local vars="GCRG GCG GCTR GCNTR"
foreach v in `vars' {
use MONA_GCRG, clear
if("`v'"=="GCRG") {
keep if mneumonic=="`v'"
drop mneumonic
rename pct_final_t `v'_actual
rename pct_first_t `v'_forecast
save GCRG_actual_forecast, replace
}
else {
keep if mneumonic=="`v'"
drop mneumonic
rename pct_final_t `v'_actual
rename pct_first_t `v'_forecast
merge 1:1 arrangementnumber countryname using GCRG_actual_forecast
drop _merge
save GCRG_actual_forecast, replace
}
}

* Removing 4sd outliers:
local vars="GCRG GCG GCTR GCNTR"
use MONA_regression, clear
foreach v in `vars' {
quietly: sum `v'
gen _mean=r(mean)
gen _sd=r(sd)
gen min_mean=_mean-4*_sd
gen max_mean=_mean+4*_sd
drop if `v'==.
drop if `v'<min_mean|`v'>max_mean
drop _mean _sd min_mean max_mean
}

* Remove all outliers for dependent variable after one full outlier run:
egen mean_GCRG=mean(GCRG)
egen sd_GCRG=sd(GCRG)
drop if abs(GCRG)>4*sd_GCRG

* LIC/Non-LIC selection:
*keep if LIC==1

* Merge regression samples with MONA data forecasts and actual values:
keep arrangementnumber countryname
merge 1:1 arrangementnumber countryname using GCRG_actual_forecast
keep if _merge==3
drop _merge

* Run Mincer-Zarnowitz regressions:
local vars="GCRG GCG GCTR GCNTR"
local counter=0
foreach v in `vars' {
local counter=`counter'+1
rename `v'_forecast forecast
* F-test:
reg `v'_actual forecast crisis08 africa america asia, robust
est store GCRG`counter'
test (_b[_cons]=0) (_b[forecast]=1)
local Fval=r(F)
local pval_F=r(p)
* Holden-Peel test:
gen ferror=`v'_actual-forecast
reg ferror crisis08 africa america asia, robust
mat cov=e(V)
mat b=e(b)
local b1=b[1,5]
local varb1=cov[5,5]
local seb1=sqrt(`varb1')
local tstatb1=`b1'/`seb1'
local pvalb1=tprob(e(df_r), abs(`tstatb1'))
drop ferror
* Results output:
outreg2 [GCRG`counter'] using MZreg_GCRG.xls, adds(F-test (a = 0 & b = 1), `Fval', ///
p-val (a = 0 & b = 1), `pval_F', Holden-Peel t-Test, `tstatb1', p-val, `pvalb1') excel bdec(3) noaster
rename forecast `v'_forecast 
}

* Erasing generated files:
erase MZreg_GCRG.xls
erase MZreg_GCRG.txt
erase GCRG_actual_forecast.dta
erase MONA_GCRG.dta


***************************************************************************************
* Table 8a: Correct and Incorrect Forecasts of Government Expenditure Growth and Its Subcomponents 
***************************************************************************************
* Prepare MONA data (forecasts and actual growth rates) for merge with regression samples:
use MONA_cleaned, clear
keep if mneumonic=="GCENL"|mneumonic=="GCEI"|mneumonic=="GCCNE"|mneumonic=="GCCENL"
gen pct_final_t=(t_end-tminus1_end)/tminus1_end if months>=18
gen pct_first_t=(t-tminus1)/tminus1 if months>=18
keep arrangementnumber countryname mneumonic pct_final_t pct_first_t
save MONA_GCENL, replace

local vars="GCENL GCEI GCCNE GCCENL"
foreach v in `vars' {
use MONA_GCENL, clear
if("`v'"=="GCENL") {
keep if mneumonic=="`v'"
drop mneumonic
rename pct_final_t `v'_actual
rename pct_first_t `v'_forecast
save GCENL_actual_forecast, replace
}
else {
keep if mneumonic=="`v'"
drop mneumonic
rename pct_final_t `v'_actual
rename pct_first_t `v'_forecast
merge 1:1 arrangementnumber countryname using GCENL_actual_forecast
drop _merge
save GCENL_actual_forecast, replace
}
}

* Removing 4sd outliers:
local vars="GCENL GCEI GCCNE GCCENL"
use MONA_regression, clear
foreach v in `vars' {
quietly: sum `v'
gen _mean=r(mean)
gen _sd=r(sd)
gen min_mean=_mean-4*_sd
gen max_mean=_mean+4*_sd
drop if `v'==.
drop if `v'<min_mean|`v'>max_mean
drop _mean _sd min_mean max_mean
}

* LIC/Non-LIC selection:
*keep if LIC==0
save GCENLreghelper, replace

* Forecast vs. actual comparisons:
local vars="GCENL GCEI GCCNE GCCENL"
local counter=0
foreach v in `vars' {
* Merge regression samples with MONA data forecasts and actual values:
use GCENLreghelper, clear
keep arrangementnumber countryname
merge 1:1 arrangementnumber countryname using GCENL_actual_forecast
keep if _merge==3
drop _merge

keep `v'_actual `v'_forecast
drop if `v'_actual==.
drop if `v'_forecast==.

local counter=`counter'+1 
gen counter=`counter'
gen pos_pos=0
gen pos_neg=0
gen neg_pos=0
gen neg_neg=0
replace pos_pos=1 if `v'_forecast>0 & `v'_actual>0
replace pos_neg=1 if `v'_forecast>0 & `v'_actual<=0
replace neg_pos=1 if `v'_forecast<=0 & `v'_actual>0
replace neg_neg=1 if `v'_forecast<=0 & `v'_actual<=0
egen pos_pos_sum=sum(pos_pos)
egen pos_neg_sum=sum(pos_neg)
egen neg_pos_sum=sum(neg_pos)
egen neg_neg_sum=sum(neg_neg)
gen total=pos_pos_sum+pos_neg_sum+neg_pos_sum+neg_neg_sum
gen pos_pos_perc=(pos_pos_sum/total)*100
gen pos_neg_perc=(pos_neg_sum/total)*100
gen neg_pos_perc=(neg_pos_sum/total)*100
gen neg_neg_perc=(neg_neg_sum/total)*100
keep pos_pos_sum neg_neg_sum pos_neg_sum neg_pos_sum pos_pos_perc neg_neg_perc pos_neg_perc ///
neg_pos_perc counter
gen mneumonic="`v'"
duplicates drop
order pos_pos_sum neg_neg_sum pos_neg_sum neg_pos_sum pos_pos_perc neg_neg_perc ///
pos_neg_perc neg_pos_perc counter

* Order for 2x2 contignency table Chi-Square test:
if("`v'"=="GCENL") {
save forecasteval, replace
}
if("`v'"!="GCENL") {
append using forecasteval
save forecasteval, replace
}
else {
continue
}
}
* Output results:
use forecasteval, clear
order counter mneumonic
sort counter
browse
* Erasing generated files:
erase forecasteval.dta
erase GCENLreghelper.dta
erase GCENL_actual_forecast.dta
erase MONA_GCENL.dta


***************************************************************************************
* Table 8b: Correct and Incorrect Forecasts of Government Revenue Growth and Its Subcomponents 
***************************************************************************************
* Prepare MONA data (forecasts and actual growth rates) for merge with regression samples:
use MONA_cleaned, clear
keep if mneumonic=="GCRG"|mneumonic=="GCG"|mneumonic=="GCTR"|mneumonic=="GCNTR"
gen pct_final_t=(t_end-tminus1_end)/tminus1_end if months>=18
gen pct_first_t=(t-tminus1)/tminus1 if months>=18
keep arrangementnumber countryname mneumonic pct_final_t pct_first_t
save MONA_GCRG, replace

local vars="GCRG GCG GCTR GCNTR"
foreach v in `vars' {
use MONA_GCRG, clear
if("`v'"=="GCRG") {
keep if mneumonic=="`v'"
drop mneumonic
rename pct_final_t `v'_actual
rename pct_first_t `v'_forecast
save GCRG_actual_forecast, replace
}
else {
keep if mneumonic=="`v'"
drop mneumonic
rename pct_final_t `v'_actual
rename pct_first_t `v'_forecast
merge 1:1 arrangementnumber countryname using GCRG_actual_forecast
drop _merge
save GCRG_actual_forecast, replace
}
}

* Removing 4sd outliers:
local vars="GCRG GCG GCTR GCNTR"
use MONA_regression, clear
foreach v in `vars' {
quietly: sum `v'
gen _mean=r(mean)
gen _sd=r(sd)
gen min_mean=_mean-4*_sd
gen max_mean=_mean+4*_sd
drop if `v'==.
drop if `v'<min_mean|`v'>max_mean
drop _mean _sd min_mean max_mean
}

* Remove all outliers for dependent variable after one full outlier run:
egen mean_GCRG=mean(GCRG)
egen sd_GCRG=sd(GCRG)
drop if abs(GCRG)>4*sd_GCRG

* LIC/Non-LIC selection:
*keep if LIC==0
save GCRGreghelper, replace

* Forecast vs. actual comparisons:
local vars="GCRG GCG GCTR GCNTR"
local counter=0
foreach v in `vars' {
* Merge regression samples with MONA data forecasts and actual values:
use GCRGreghelper, clear
keep arrangementnumber countryname
merge 1:1 arrangementnumber countryname using GCRG_actual_forecast
keep if _merge==3
drop _merge

keep `v'_actual `v'_forecast
drop if `v'_actual==.
drop if `v'_forecast==.

local counter=`counter'+1 
gen counter=`counter'
gen pos_pos=0
gen pos_neg=0
gen neg_pos=0
gen neg_neg=0
replace pos_pos=1 if `v'_forecast>0 & `v'_actual>0
replace pos_neg=1 if `v'_forecast>0 & `v'_actual<=0
replace neg_pos=1 if `v'_forecast<=0 & `v'_actual>0
replace neg_neg=1 if `v'_forecast<=0 & `v'_actual<=0
egen pos_pos_sum=sum(pos_pos)
egen pos_neg_sum=sum(pos_neg)
egen neg_pos_sum=sum(neg_pos)
egen neg_neg_sum=sum(neg_neg)
gen total=pos_pos_sum+pos_neg_sum+neg_pos_sum+neg_neg_sum
gen pos_pos_perc=(pos_pos_sum/total)*100
gen pos_neg_perc=(pos_neg_sum/total)*100
gen neg_pos_perc=(neg_pos_sum/total)*100
gen neg_neg_perc=(neg_neg_sum/total)*100
keep pos_pos_sum pos_neg_sum neg_pos_sum neg_neg_sum pos_pos_perc pos_neg_perc ///
neg_pos_perc neg_neg_perc counter
gen mneumonic="`v'"
duplicates drop

* Order for 2x2 contignency table Chi-Square test:
order pos_pos_sum neg_neg_sum neg_pos_sum pos_neg_sum pos_pos_perc neg_neg_perc ///
pos_neg_perc neg_pos_perc counter

if("`v'"=="GCRG") {
save forecasteval, replace
}
if("`v'"!="GCRG") {
append using forecasteval
save forecasteval, replace
}
else {
continue
}
}
* Output results:
use forecasteval, clear
order counter mneumonic
sort counter
browse
* Erasing generated files:
erase forecasteval.dta
erase GCRGreghelper.dta
erase GCRG_actual_forecast.dta
erase MONA_GCRG.dta


***************************************************************************************
* Table 9: Contributors to Government Budget Growth Forecast Errors
***************************************************************************************
* Panel A: Government Expenditure
*********************************
* Removing 4sd outliers:
local vars="GCENL GCEI GCCNE GCCENL"
use MONA_regression, clear
foreach v in `vars' {
quietly: sum `v'
gen _mean=r(mean)
gen _sd=r(sd)
gen min_mean=_mean-4*_sd
gen max_mean=_mean+4*_sd
drop if `v'==.
drop if `v'<min_mean|`v'>max_mean
drop _mean _sd min_mean max_mean
}

* LIC/Non-LIC selection:
*keep if LIC==0

* Output results:
reg GCENL GCEI GCCNE GCCENL, robust
outreg2 using MONAreg_GCENL_SD4.xls, excel bdec(3)

* Erase generated files:
erase MONAreg_GCENL_SD4.xls 
erase MONAreg_GCENL_SD4.txt 

* Panel B: Government Revenue
*****************************
* Removing 4sd outliers:
local vars="GCRG GCG GCTR GCNTR"
use MONA_regression, clear
foreach v in `vars' {
quietly: sum `v'
gen _mean=r(mean)
gen _sd=r(sd)
gen min_mean=_mean-4*_sd
gen max_mean=_mean+4*_sd
drop if `v'==.
drop if `v'<min_mean|`v'>max_mean
drop _mean _sd min_mean max_mean
}

* Wipe out all outliers for dependent variable after one full outlier run:
egen mean_GCRG=mean(GCRG)
egen sd_GCRG=sd(GCRG)
drop if abs(GCRG)>4*sd_GCRG

* LIC/Non-LIC selection:
*keep if LIC==0

* Output results:
reg GCRG GCG GCTR GCNTR, robust
outreg2 using MONAreg_GCRG_SD4.xls, excel bdec(3)

* Erase generated files:
erase MONAreg_GCRG_SD4.xls 
erase MONAreg_GCRG_SD4.txt 


********************************************************************************
* Erase files created at beginning of file:
erase MONA_regression.dta
erase MONA_cleaned.dta
