********************************************************************************
* FORECASTS IN TIMES OF CRISES -- MONA data cleaning:
********************************************************************************

cd "insert folder"

* Loading in raw MONA data:
import excel using Mecon.xlsx, firstrow clear case(lower)

* Renaming
rename t3 tminus3
rename t2 tminus2
rename t1 tminus1
rename z tplus1
rename aa tplus2 
rename ab tplus3
rename t4 tplus4

* Dropping observations with all zero entries:
drop if tminus3==0 & tminus2==0 & tminus1==0 & t==0 & tplus1==0 & tplus2==0 & tplus3==0 & tplus4==0

* Dropping observations with all zero entries in t, t+1, t+2, t+3 and t+4:
drop if t==0 & tplus1==0 & tplus2==0 & tplus3==0 & tplus4==0

* Dropping observations with all missing entries:
drop if tminus3==. & tminus2==. & tminus1==. & t==. & tplus1==. & tplus2==. & tplus3==. & tplus4==.

* Dropping observations with all missing entries in t, t+1, t+2, t+3 and t+4:
drop if t==. & tplus1==. & tplus2==. & tplus3==. & tplus4==.

* Dropping variables that are not needed:
drop durationofannualarrangementf durationofannualarrangementt yeartype fystartmonth ///
fyendmonth countrycode approvalyear initialendyear sort programtype programyear

* Correcting a few naming errors in the database:
drop if mneumonic=="DG_S"
replace mneumonic="BFOL_G_L" if mneumonic=="BFOLG_L"
replace mneumonic="BMII" if mneumonic=="BMII "
replace mneumonic="NXG_R" if mneumonic=="BNXG_R  "
replace mneumonic="NXG_R" if mneumonic=="NXG_R  "  
save helper, replace

* Merging data from first and last review:
use helper, clear
keep if reviewsequence=="L   "
keep arrangementnumber countryname intialenddate revisedenddate tminus3 tminus2 tminus1 ///
t tplus1 tplus2 tplus3 tplus4 mneumonic indicatorcurrency description arrangementtype
rename tminus3 tminus3_end
rename tminus2 tminus2_end
rename tminus1 tminus1_end
rename t t_end
rename tplus1 tplus1_end
rename tplus2 tplus2_end
rename tplus3 tplus3_end
rename tplus4 tplus4_end
gen enddate=intialenddate
replace enddate=revisedenddate if revisedenddate>intialenddate & revisedenddate!=.
rename indicatorcurrency indicatorcurrency_end
duplicates tag arrangementnumber mneumonic countryname, gen(dup)
drop if dup==1 & description=="Interest, accrued"
drop dup
save enddate, replace

use helper, clear
keep if reviewsequence=="E   "
rename approvaldate startdate
duplicates drop
merge 1:1 arrangementnumber countryname mneumonic description using enddate
keep if _merge==3
drop reviewsequence reviewtype _merge boardactiondate revisedenddate intialenddate
save helper, replace

* Dropping all programs that ran less than 18 months:
use helper, clear
format startdate %td
gen double startmonth = mofd(startdate)
gen double startyear = yofd(startdate)
gen double month = month(startdate)
format enddate %td
gen double endmonth = mofd(enddate)
gen months=endmonth-startmonth
drop if months<18

* Generate 2008 crisis dummy
gen crisis08=0
replace crisis08=1 if startyear==2008
drop startmonth startdate endmonth enddate
save helper, replace

* Drop observation for which the first and last 'period t' entry is unchanged, i.e. 
* forecasts were most likely not replaced with actual realizations:
use helper, clear
drop if t==t_end 
drop if tplus1==tplus1_end & months>=30
drop if tplus2==tplus2_end & months>=42 

* Replace forecasts and actual values that have a 0 with a missing value
local vars="tminus3 tminus2 tminus1 t tplus1 tplus2 tplus3 tplus4 tminus3_end tminus2_end " ///
		+ "tminus1_end t_end tplus1_end tplus2_end tplus3_end tplus4_end"
foreach k in `vars' {
replace `k'=. if `k'==0
}

* Generate continental dummy variables for countries in sample;
* CAREFUL: only correctly generates dummies for countries in our sample, not others:
gen europe=0
gen asia=0
gen america=0
gen africa=0

* Europe:
local europe=`" "ALBANIA" "BOSNIA AND HERZEGOVINA " "ARMENIA" "BULGARIA" "CROATIA""' ///
		   + `""CYPRUS" "GREECE" "KOSOVO, REPUBLIC OF" "LATVIA" "MACEDONIA (FYR)""' ///
		   + `""MOLDOVA" "PORTUGAL" "ROMANIA" "SERBIA,REPUBLIC OF" "TURKEY" "UKRAINE""'
foreach e in `europe' {
replace europe=1 if countryname=="`e'"
} 

* Asia:
local asia=`""AFGHANISTAN,ISLAMIC REPUBLIC OF" "BANGLADESH" "GEORGIA" "IRAQ" "JORDAN" "' ///
		 + `""KYRGYZ REPUBLIC" "MALDIVES" "MONGOLIA" "NEPAL" "PAKISTAN" "SOLOMON ISLANDS""' ///
		 + `""SRI LANKA" "TAJIKISTAN" "'
foreach a in `asia' {
replace asia=1 if countryname=="`a'"
}

* Africa:
local africa=`""ANGOLA" "BENIN" "BURKINA FASO" "BURUNDI" "CAMEROON" "CAPE VERDE""' ///
		   + `""CENTRAL AFRICAN REPUBLIC" "CHAD" "COMOROS" "CONGO, REPUBLIC OF  ""' ///
		   + `""CONGO,DEMOCRATIC REPUBLIC OF"  "DJIBOUTI" "GABON" "GAMBIA, THE ""' ///
		   + `""GHANA" "GUINEA" "GUINEA-BISSAU" "KENYA" "LESOTHO" "LIBERIA" "MADAGASCAR""' ///
		   + `""MALAWI" "MALI" "MAURITANIA" "MOROCCO" "MOZAMBIQUE" "NIGER" "NIGERIA""' ///
		   + `""RWANDA" "SAO TOME AND PRINCIPE" "SENEGAL" "SEYCHELLES" "SIERRA LEONE""' ///
		   + `""TANZANIA" "TOGO" "TUNISIA" "UGANDA" "ZAMBIA""' 
foreach a in `africa' {
replace africa=1 if countryname=="`a'"
}
replace africa=1 if countryname=="COTE D'IVOIRE"

* America:
local america=`""ANTIGUA AND BARBUDA" "ARGENTINA" "BOLIVIA" "BRAZIL" "COLOMBIA""' ///
			+ `""DOMINICA" "DOMINICAN REPUBLIC" "EL SALVADOR" "GRENADA" "HAITI""' ///
			+ `""HONDURAS" "NICARAGUA" "PARAGUAY" "PERU" "ST. KITTS AND NEVIS" "URUGUAY""'
foreach a in `america' {
replace america=1 if countryname=="`a'"
}

* Cleaning up discrepancies in currency names:
label variable indicatorcurrency_end "Indicator Currency End"

gen indicatorcurrency_helper="."
gen indicatorcurrency_end_helper="."
replace indicatorcurrency_helper="NCU (billions)" if indicatorcurrency=="Billions" & indicatorcurrency_end=="Billions" 
replace indicatorcurrency_end_helper="NCU (billions)" if indicatorcurrency=="Billions" & indicatorcurrency_end=="Billions" 
replace indicatorcurrency_helper="NCU (billions)" if indicatorcurrency=="Billions" & indicatorcurrency_end=="NCU (billions)" 
replace indicatorcurrency_end_helper="NCU (billions)" if indicatorcurrency=="NCU (billions)" & indicatorcurrency_end=="Billions" 
replace indicatorcurrency_helper="US$ (billions)" if indicatorcurrency=="Billions" & indicatorcurrency_end=="US$ (billions)" 
replace indicatorcurrency_end_helper="NCU/US$" if indicatorcurrency=="NCU/US$" & indicatorcurrency_end=="None" 
replace indicatorcurrency_helper="NCU/US$" if indicatorcurrency=="None" & indicatorcurrency_end=="NCU/US$" 
replace indicatorcurrency_end_helper="Index Number" if indicatorcurrency=="Index Number" & indicatorcurrency_end=="None" 
replace indicatorcurrency_helper="Index Number" if indicatorcurrency=="None" & indicatorcurrency_end=="Index Number" 
drop if indicatorcurrency=="NCU (billions)" & indicatorcurrency_end==" US$ (millions)" 
drop if indicatorcurrency=="US$ (billions)" & indicatorcurrency_end=="NCU (billions)" 
replace indicatorcurrency=indicatorcurrency_helper if indicatorcurrency_helper!="."
replace indicatorcurrency_end=indicatorcurrency_end_helper if indicatorcurrency_end_helper!="."
drop indicatorcurrency_helper indicatorcurrency_end_helper

* Cleaning up data entry errors with regard to imports -- some desk economists enter 
* imports as negative numbers:
local vars="NM BMG BMS" 

foreach v in `vars' {
replace t_end=abs(t_end) if mneumonic=="`v'"
replace tminus1_end=abs(tminus1_end) if mneumonic=="`v'"
replace t=abs(t) if mneumonic=="`v'"
replace tminus1=abs(tminus1) if mneumonic=="`v'" 
}
save MONA_cleaned, replace


********************************************************************************
* Computing forecast errors: forecast error = forecast - actual (in growth rates)
********************************************************************************
* Note: 
* t_end is the actual value at time t at the end of the program
* tminus1_end is the actual value at time t-1 at the end of the program
* t is the forecasted value at time t at the beginnning of the program
* tminus1 is the forecasted value at time t-1 at the beginnning of the program
* fcast_error_t_ln>0 implies actual growth rate is greater than forecasted growth rate,
* fcast_error_t_ln<0 implies actual growth rate is smaller than forecasted growth rate
use MONA_cleaned, clear
gen pct_final_t=(t_end-tminus1_end)/tminus1_end if months>=18
gen pct_first_t=(t-tminus1)/tminus1 if months>=18
gen fcast_error_t_pct=pct_first_t-pct_final_t if months>=18
drop pct_final_t pct_first_t 

* Dropping irrelevant variables 
drop indicatorcurrency_end
rename month startmonth

save MONA_fcasts, replace 

* Erase not needed datasets:
erase enddate.dta
erase helper.dta


